/*
 * File   : init.ec
 * Purpose: This file initializes the database (Informix).
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1998, 1999 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <unistd.h>

#include <Xmt/Xmt.h>
#include "platten.h"
EXEC SQL include sqlca;
EXEC SQL include sqlda;
EXEC SQL include sqltypes;

EXEC SQL define SUCCESS 0;
EXEC SQL define WARNING 1;
EXEC SQL define NODATA 100;
EXEC SQL define RTERROR -1;

int has_trans;

void init ()
{
	EXEC SQL database disk;

	if (SQLCODE != 0)
	   DisplaySqlError (toplevel);

	has_trans = (sqlca.sqlwarn.sqlwarn1 == 'W');
}

void CloseDatabase ()
{
	EXEC SQL disconnect current;
}

int CheckSqlError ()
{
int err_code = RTERROR;

	if (SQLSTATE[0] == '0')		/* trap '00', '01', '02' */
	{
	   switch (SQLSTATE[1])
	   {
	      case '0': err_code = SUCCESS; break;
	      case '1': err_code = WARNING; break;
	      case '2': err_code = NODATA; break;
	   }
	}

	return (err_code);
}

void DisplaySqlError (Widget w)
{
EXEC SQL BEGIN DECLARE SECTION;
	int exception_count;
	char overflow[2];
	int exception_num = 1;
	char class_id[255];
	char subclass_id[255];
	char message[255];
	int messlen;
	char sqlstate_code[6];
	int i;
EXEC SQL END DECLARE SECTION;
int j;
char hv0[128], hv1[128], hv3[128], BigMessage[1024];

	sprintf (hv0, "SQLSTATE  : %s\nSQLCODE   : %d\n\n",
		SQLSTATE, SQLCODE);

	EXEC SQL get diagnostics :exception_count = NUMBER, :overflow = MORE;
	sprintf (hv1, "EXCEPTIONS: Number=%d   More? %s\n",
		exception_count, overflow);

	sprintf (BigMessage, "%s%s", hv0, hv1);

	for (i = 1; i <= exception_count; i++)
	{
	   EXEC SQL get diagnostics exception :i
		:sqlstate_code = RETURNED_SQLSTATE,
		:class_id = CLASS_ORIGIN, :subclass_id = SUBCLASS_ORIGIN,
		:message = MESSAGE_TEXT, :messlen = MESSAGE_LENGTH;

	   strcat (BigMessage, "--------------------------------\n");
	   sprintf (hv3, "EXCEPTION %d: SQLSTATE=%s\n", i, sqlstate_code);
	   message[messlen-1] = 0;
	   strcat (BigMessage, hv3);
	   sprintf (hv3, "MESSAGE TEXT: %s\n", message);
	   strcat (BigMessage, hv3);
	   j = byleng (class_id, stleng (class_id));
	   class_id[j] = 0;
	   sprintf (hv3, "CLASS ORIGIN: %s\n", class_id);
	   strcat (BigMessage, hv3);
	   j = byleng (subclass_id, stleng (subclass_id));
	   subclass_id[j] = 0;
	   sprintf (hv3, "SUBCLASS ORIGIN: %s\n", subclass_id);
	   strcat (BigMessage, hv3);
	}

	XmtDisplayError (w, NULL, BigMessage);
}

int CheckDate (Widget w, char *date)
{
int i;
char hv0[128];
long idate;

	if ((i = rdefmtdate (&idate, app_resources.datePattern, date)) != 0)
	{
	   switch (i)
	   {
	      case -1204: XmtDisplayError (w, "iError1204", NULL); break;
	      case -1205: XmtDisplayError (w, "iError1205", NULL); break;
	      case -1206: XmtDisplayError (w, "iError1206", NULL); break;
	      case -1209: XmtDisplayError (w, "iError1209", NULL); break;
	      case -1212: XmtDisplayError (w, "iError1212", NULL); break;

	      default:
		 sprintf (hv0, "An unknown error %d occured while transforming a date string into an internal value. Please contact the author via email at theofilu@eunet.at!", i);
		 XmtDisplayError (w, "iUnknown", hv0);
	   }

	   return i;
	}

	return 0;
}

char *IDateToChar (long idate, char *sdate)
{
	if (idate < 1000)
	{
	   *sdate = 0;
	   return sdate;
	}

	rfmtdate (idate, app_resources.datePattern, sdate);
	return sdate;
}

long CharToIDate (Widget w, char *date)
{
int i;
char hv0[128];
long idate;

	if ((i = rdefmtdate (&idate, app_resources.datePattern, date)) != 0)
	{
	   switch (i)
	   {
	      case -1204: XmtDisplayError (w, "iError1204", NULL); break;
	      case -1205: XmtDisplayError (w, "iError1205", NULL); break;
	      case -1206: XmtDisplayError (w, "iError1206", NULL); break;
	      case -1209: XmtDisplayError (w, "iError1209", NULL); break;
	      case -1212: XmtDisplayError (w, "iError1212", NULL); break;

	      default:
		 sprintf (hv0, "An unknown error %d occured while transforming a date string into an internal value. Please contact the author via email at theofilu@eunet.at!", i);
		 XmtDisplayError (w, "iUnknown", hv0);
	   }

	   return 0L;
	}

	return idate;
}

char *GetCompoundString (XmString cstr, char *ret, int retlen)
{
XmStringContext ck;
#if XmVersion < 2000
   char *zeichen, *tag;
   XmStringDirection dir;
   Boolean sep, status;
#else
   XmStringComponentType ct;
   unsigned int len;
   XtPointer value;
#endif
char hv0[128];

	*ret = 0;
	/*
	 * Kontext zum Auslesen initialisieren.
	 */
	if (!XmStringInitContext (&ck, cstr))
	{
	   XmtDisplayError (toplevel, "mErrorInitContext", NULL);
	   return NULL;
	}

	do
	{
#if XmVersion < 2000
	   status = XmStringGetNextSegment (ck, &zeichen, &tag, &dir, &sep);
#else
	   ct = XmStringGetNextTriple (ck, &len, &value);
/***************
	   switch (ct)
	   {
	      case XmSTRING_COMPONENT_UNKNOWN:
		printf ("XmSTRING_COMPONENT_UNKNOWN: \n"); break;
	      case XmSTRING_COMPONENT_CHARSET:
		printf ("XmSTRING_COMPONENT_CHARSET: %s\n", value); break;
	      case XmSTRING_COMPONENT_TEXT:
		printf ("XmSTRING_COMPONENT_TEXT: %s\n", value); break;
	      case XmSTRING_COMPONENT_DIRECTION:
		printf ("XmSTRING_COMPONENT_DIRECTION:\n"); break;
	      case XmSTRING_COMPONENT_SEPARATOR:
		printf ("XmSTRING_COMPONENT_SEPARATOR:\n"); break;
	      case XmSTRING_COMPONENT_TAB:
		printf ("XmSTRING_COMPONENT_TAB:\n"); break;
	      case XmSTRING_COMPONENT_LOCALE:
		printf ("XmSTRING_COMPONENT_LOCALE: %s\n", value); break;
	      case XmSTRING_COMPONENT_LOCALE_TEXT:
		printf ("XmSTRING_COMPONENT_LOCALE_TEXT(%d): %s\n", len, value); break;
	      case XmSTRING_COMPONENT_WIDECHAR_TEXT:
		printf ("XmSTRING_COMPONENT_WIDECHAR_TEXT:\n"); break;
	      case XmSTRING_COMPONENT_RENDITION_BEGIN:
		printf ("XmSTRING_COMPONENT_RENDITION_BEGIN: %s\n",value); break;
	      case XmSTRING_COMPONENT_RENDITION_END:
		printf ("XmSTRING_COMPONENT_RENDITION_END: %s\n",value); break;
	      case XmSTRING_COMPONENT_LAYOUT_PUSH:
		printf ("XmSTRING_COMPONENT_LAYOUT_PUSH: %s\n",value); break;
	      case XmSTRING_COMPONENT_LAYOUT_POP:
		printf ("XmSTRING_COMPONENT_LAYOUT_POP: %s\n",value); break;
	   }
******************/
	   if (ct == XmSTRING_COMPONENT_WIDECHAR_TEXT)
	   {
	      size_t len;

	      len = wcstombs (hv0, (wchar_t *)value, sizeof (hv0));

	      if (len == (size_t)-1)
	      {
	         XmStringFreeContext (ck);
	         XtFree (value);
	         XmtDisplayError (toplevel, "mErrorWchar", NULL);
	         return NULL;
	      }

	      hv0[len] = 0;

	      if ((strlen (hv0) + strlen (ret)) <= retlen)
	         strcat (ret, hv0);
	   }

	   if (ct == XmSTRING_COMPONENT_TEXT ||
	       ct == XmSTRING_COMPONENT_LOCALE_TEXT)
	   {
	      if (len > 0)
	      {
		 strncpy (hv0, (String)value, sizeof (hv0));
		 hv0[sizeof(hv0)-1] = 0;

		 if ((strlen (hv0) + strlen (ret)) <= retlen)
	            strcat (ret, hv0);
	      }
	   }

/*	   if (ct == XmSTRING_COMPONENT_TAB)
	      strcat (ret, "\t");
*/
	   if (ct == XmSTRING_COMPONENT_SEPARATOR)
	      strcat (ret, "\n");
#endif
#if XmVersion < 2000
	   if (sep && status)
	      strcat (ret, "\n");
	   else if (status)
	   {
printf ("Teil: %s\n", zeichen);
	      strncpy (hv0, zeichen, sizeof (hv0));
	      hv0[sizeof(hv0)-1] = 0;

	      if ((strlen (hv0) + strlen (ret)) <= retlen)
	         strcat (ret, hv0);

	     XtFree (zeichen);
	     XtFree (tag);
	   }
#else
	   XtFree (value);
#endif
	}
#if XmVersion < 2000
	while (status);
#else
	while (ct != XmSTRING_COMPONENT_END);
#endif

	XmStringFreeContext (ck);
	return ret;
}

#if XmVersion < 2000
XmString StringTableToXmString (XmStringTable tbl, int anz, XmString sep)
{
int i;
XmString str, str1, str2;
char *b;

	for (i = 0; i < anz; i++)
	{
	   XmStringGetLtoR (tbl[i], XmFONTLIST_DEFAULT_TAG, &b);
	   printf ("Eintrag: %s\n", b);
	   XtFree (b);

	   if (i == 0)
	      str = XmStringCopy (tbl[i]);
	   else
	   {
	      str2 = XmStringConcat (str, sep);
	      XmStringFree (str);
	      str1 = XmStringConcat (str2, tbl[i]);
	      XmStringFree (str2);
	   }

	   if (i > 0)
	   {
	      str = XmStringCopy (str1);
	      XmStringFree (str1);
	   }
	}

	return str;
}
#endif

/*
 * Folgende Funktion liest den Inhalt der Datei $HOME/.platten aus
 * und setzt den Inhalt in die Struktur 'option_t'. Die Routine wird
 * beim Start des Programms aufgerufen.
 */
Bool GetOptions (Widget w)
{
char *envHome, hv0[256], hv1[512], *p;
int fd;
struct stat st;

	if ((envHome = getenv ("HOME")) == NULL)
	{
	   XmtDisplayError (w, "ErrorOptionEnv", NULL);
	   return False;
	}

	sprintf (hv0, "%s/.platten", envHome);

	if ((fd = open (hv0, O_RDWR | O_CREAT, 0600)) == -1)
	{
	   XmtDisplayError (toplevel, "ErrorOptionFile", NULL);
	   XmtDisplayDefaultCursor (toplevel);
	   return False;
	}

	if (fstat (fd, &st) == -1)
	{
	   close (fd);
	   XmtDisplayError (w, "ErrorOptionStat", NULL);
	   return False;
	}

	if (st.st_size > 1L)
	{
	   while (ReadLine (fd, hv1, sizeof (hv1)) != -1)
	   {
	      if ((p = strchr (hv1, '=')) != NULL)
	      {
	         *p = 0;
	         p++;
		 ltrim (p);

		 if (!strcasecmp (hv1, "Chooser"))
		    option.chPrinter = atoi (p);

	         if (!strcasecmp (hv1, "Printer"))
	            strcpy (option.ipPrinter, p);

	         if (!strcasecmp (hv1, "File"))
		    strcpy (option.ipFile, p);

	         if (!strcasecmp (hv1, "Separator"))
	         {
		    option.ipTrenn[0] = *p;
		    option.ipTrenn[1] = 0;
	         }

	         if (!strcasecmp (hv1, "Dateformat"))
		    strcpy (option.ipDatef, p);
	      }
	   }
	}
	else
	{
	   memset ((void *)&option, 0, sizeof (option_t));
	   XmtDisplayInformation (w, "InfoOptionFile", NULL, NULL);
	   sprintf (hv1, "Version=%s\nChooser=0\nPrinter=lpr\nFile=\nSeparator=|\nDateformat=dd.mm.yyyy\n", app_resources.Version);
	   write (fd, hv1, strlen (hv1));
	   strcpy (option.ipPrinter, "lpr");
	   strcpy (option.ipTrenn, "|");
	   strcpy (option.ipDatef, "dd.mm.yyyy");
	}

	close (fd);
        return True;
}

/* ReadLine:
 *    Reads one line from a file. This function assumes that the file
 *    is an ASCII file and contains some kind of text where the lines
 *    are separated with line-feed characters (0x0a).
 *    If a line is longer than the defined length of the variable,
 *    it truncates the line to the defined length.
 *    If the end of file is reached, it returns -1 to indicate this.
 */
int ReadLine (int fd, char *buf, int len)
{
char ch;
int i = 0;

        while (read (fd, (char *)&ch, 1) == 1)
        {
           if (ch == '\n')
           {
	      if (i < len)
                 *(buf+i) = 0;
	      else
	         *(buf+(len-1)) = 0;

              return i;
           }

           if (i < len)
              *(buf+i) = ch;

           i++;
        }

	if (i < len)
           *(buf+i) = 0;
	else
	   *(buf+(len-1)) = 0;

        return -1;
}

/* ltrim:
 *    This function strips the blanks in front of a line. However, it stops
 *    after the first non blank character.
 */
char *ltrim (char *str)
{
char *p, *ret, *sp;
int flag = 0;

        if ((p = strdup (str)) == NULL)
           return str;

        ret = str;
        sp = p;

        while (*p != 0)
        {
           if (*p != ' ')
              flag = 1;

           if (flag)
              *str++ = *p;
           p++;
        }

        *str = 0;
        free (sp);
        return ret;
}

/*
 * Folgende Funktion schneidet alle Leerzeichen am Ende eines Strings ab.
 */
char *rtrim (char *buf)
{
char *p;
int i;

	i = strlen (buf);

	if (i <= 0)
	   return buf;

	p = buf + (i - 1);

	while (*p == ' ' && p >= buf)
	   p--;

	*(p+1) = '\0';
	return buf;
}

