/*
 * File   : options.c
 * Purpose: This file contains the routines to set some options.
 * Autor  : Andreas Theofilu
 *
 * Copyright (C) 1998, 1999 by Theos Soft.
 */

/* 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <unistd.h>

#include <Xmt/Xmt.h>
#include <Xmt/Create.h>
#include <Xmt/Converters.h>
#include <Xmt/Procedures.h>
#include <Xmt/WidgetType.h>
#include <Xmt/Dialogs.h>
#include <Xmt/Chooser.h>
#include "platten.h"


XtResource option_resource[] = {
	{"chPrinter", "chPrinter", XtRInt, sizeof (int),
	 XtOffsetOf (option_t, chPrinter), XtRImmediate, NULL},
	{"ipPrinter", "ipPrinter", XmtRBuffer, XmtSizeOf (option_t, ipPrinter),
	 XtOffsetOf (option_t, ipPrinter), XtRImmediate, NULL},
	{"ipFile", "ipFile", XmtRBuffer, XmtSizeOf (option_t, ipFile),
	 XtOffsetOf (option_t, ipFile), XtRImmediate, NULL},
	{"ipTrenn", "ipTrenn", XmtRBuffer, XmtSizeOf (option_t, ipTrenn),
	 XtOffsetOf (option_t, ipTrenn), XtRImmediate, NULL},
	{"ipDatef", "ipDatef", XmtRBuffer, XmtSizeOf (option_t, ipDatef),
	 XtOffsetOf (option_t, ipDatef), XtRImmediate, NULL}
};

/*
 * Einige globale Variablen
 */
option_t option;
Widget OptionWidget = (Widget)0;

/*
 * Globale Funktionsdeklarationen die nur in diesem Modul verwendet
 * werden.
 */
int ReadLine (int fd, char *buf, int len);

/*
 * Handling der Dialogbox zur Eingabe von Singles
 */
void Options ()
{
char *envHome, hv0[256], hv1[512];
int fd;

	XmtDisplayBusyCursor (toplevel);

	if ((envHome = getenv ("HOME")) == NULL)
	{
	   XmtDisplayError (toplevel, "ErrorOptionEnv", NULL);
	   XmtDisplayDefaultCursor (toplevel);
	   return;
	}

	if (!OptionWidget)
	   OptionWidget = XmtBuildQueryDialog (toplevel, "option", 
		option_resource, XtNumber (option_resource), NULL);

	if (XmtDialogDoSync (OptionWidget, &option))
	{
	   if (option.chPrinter > 0 && !strlen (option.ipFile))
	   {
	      XmtDisplayError (toplevel, "ErrorOptionField", NULL);
	      option.chPrinter = 0;
	   }

	   if (option.chPrinter == 1 && strrchr (option.ipFile, '.') == NULL)
	      strcat (option.ipFile, ".ps");

	   if (option.chPrinter == 2 && strrchr (option.ipFile, '.') == NULL)
	      strcat (option.ipFile, ".cvs");

	   sprintf (hv0, "%s/.platten", envHome);

	   if ((fd = open (hv0, O_RDWR | O_TRUNC, 0600)) == -1)
	   {
	      XmtDisplayError (toplevel, "ErrorOptionFile", NULL);
	      XmtDisplayDefaultCursor (toplevel);
	      return;
	   }

	   sprintf (hv1, "Version=%s\nChooser=%d\nPrinter=%s\nFile=%s\nSeparator=%s\nDateformat=%s\n",
		app_resources.Version, option.chPrinter, option.ipPrinter,
		option.ipFile, option.ipTrenn, option.ipDatef);
	   write (fd, hv1, strlen (hv1));
	   close (fd);
	}

	XmtDisplayDefaultCursor (toplevel);
}

void OptionDialogInit (Widget w)
{
Widget wPrinter = XmtNameToWidget (toplevel, "*option*ipPrinter");
Widget wFile = XmtNameToWidget (toplevel, "*option*ipFile");

	switch (option.chPrinter)
	{
	   case 0: 
	      XtSetSensitive (wPrinter, True);
	      XtSetSensitive (wFile, False);
	   break;

	   case 1:
	   case 2:
	      XtSetSensitive (wPrinter, False);
	      XtSetSensitive (wFile, True);
	   break;
	}
}

void OptionChooserChangedCallback (Widget w, XtPointer tag, XtPointer data)
{
XmtChooserCallbackStruct *Chooser = (XmtChooserCallbackStruct *)data;
Widget wPrinter = XmtNameToWidget (toplevel, "*option*ipPrinter");
Widget wFile = XmtNameToWidget (toplevel, "*option*ipFile");

	switch (Chooser->item)
	{
	   case 0: 
	      XtSetSensitive (wPrinter, True);
	      XtSetSensitive (wFile, False);
	   break;

	   case 1:
	   case 2:
	      XtSetSensitive (wPrinter, False);
	      XtSetSensitive (wFile, True);
	   break;
	}
}

